/*
 * Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
 * Copyright (C) 2010 - DIGITEO - Allan SIMON
 * 
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at    
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

package org.scilab.modules.metanet;

import java.util.List;

import javax.swing.SwingUtilities;

import org.scilab.modules.metanet.configuration.ConfigurationManager;
import org.scilab.modules.metanet.graph.MetanetDiagram;


public final class Metanet {

	/**
	 * Create a diagram model with a visible window
	 * @return The new empty model
	 */
	public static MetanetDiagram createEmptyDiagram() {
		MetanetDiagram metanetDiagramm = createANotShownDiagram();
		MetanetTab.showTabFromDiagram(metanetDiagramm);
		return metanetDiagramm;
	}

	/**
	 * Create a diagram model
	 * @return The new empty model
	 */

	public static MetanetDiagram createANotShownDiagram() {
		MetanetDiagram metanetDiagramm = new MetanetDiagram();
		metanetDiagramm.installListeners();
		MetanetTab.createTabFromDiagram(metanetDiagramm);
		return metanetDiagramm;
	}

    /**
     * @return All the opened (hidden or shown) diagrams
     */
    public static List<MetanetDiagram> getDiagrams() {
	return MetanetTab.getAllDiagrams();
    }
	
	/**
	 * @param args
	 */


	// FIXME : Remove main
	public static void main(String[] args) {
		metanet();
	}

	/**
	 * Entry point with filename
	 * @param fileName The filename
	 */
	public static void metanet(String fileName) {
		final String filename = fileName;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				ConfigurationManager.getInstance().addToRecentFiles(filename);
				if (!MetanetTab.focusOnExistingFile(filename)) {
					MetanetDiagram diagram = createEmptyDiagram();
					diagram.openDiagramFromFile(filename);
					diagram.refresh();
				}
				ConfigurationManager.getInstance().saveConfig();
			}
		});
	}

	/**
	 * Entry point with filename
	 * @param fileName The filename
	 */
	public static void metanet() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MetanetDiagram diagram = createEmptyDiagram();
				}
		});
	}

	/**
	 * Close the current metanet session
	 */
	public static void closeSession() {
		List<MetanetDiagram> diagrams = MetanetTab.getAllDiagrams();

		/*
		 * We are looping in the inverted order because we have to close latest
		 * add diagrams before any others.
		 * 
		 * Furthermore the closeDiagram operation modify the diagram list.
		 */
		int i = diagrams.size() - 1;
		while (i >= 0) {
			diagrams.get(i).closeDiagram();
			i = diagrams.size() - 1;
		}

	}
	
	public static void warnObjectByUID(String uid, String message){
		// Try to find a block with given index (UID)
		List<MetanetDiagram> allDiagrams = Metanet.getDiagrams();
		for (int i = 0; i < allDiagrams.size(); ++i) {
		    allDiagrams.get(i).warnObjectByUID(uid, message);
		}
	}


    /**
     * This method is called when the user exits from Scilab
     */
    public static void closeMetanetFromScilab() {

	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		// call close on all diagrams
		while (MetanetTab.getAllDiagrams().size() > 0) {
		    MetanetTab.getAllDiagrams().get(0).closeDiagram(true);
		}
	    }
	});
    }
	
}
