/*
 * Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
 * Copyright (C) 2010 - DIGITEO - Allan SIMON
 * 
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at    
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

package org.scilab.modules.metanet.io;

import org.scilab.modules.graph.io.ScilabBooleanCodec;
import org.scilab.modules.graph.io.ScilabDoubleCodec;
import org.scilab.modules.graph.io.ScilabIntegerCodec;
import org.scilab.modules.graph.io.ScilabListCodec;
import org.scilab.modules.graph.io.ScilabObjectCodec;
import org.scilab.modules.graph.io.ScilabStringCodec;
import org.scilab.modules.metanet.edge.BasicEdge;
import org.scilab.modules.metanet.graph.MetanetDiagram;
import org.scilab.modules.metanet.node.BasicNode;
import org.scilab.modules.metanet.node.DefaultNode;
import org.scilab.modules.types.scilabTypes.ScilabBoolean;
import org.scilab.modules.types.scilabTypes.ScilabDouble;
import org.scilab.modules.types.scilabTypes.ScilabInteger;
import org.scilab.modules.types.scilabTypes.ScilabList;
import org.scilab.modules.types.scilabTypes.ScilabString;
import org.w3c.dom.Document;

import com.mxgraph.io.mxCodec;
import com.mxgraph.io.mxCodecRegistry;
import com.mxgraph.io.mxObjectCodec;

public class MetanetCodec extends mxCodec {
    /**
     * Register usefull codecs and packages for encoding/decoding diagrams
     */
    static {
	// Add all metanet packages
    	mxCodecRegistry.addPackage("org.scilab.modules.graph");
    	mxCodecRegistry.addPackage("org.scilab.modules.metanet");
    	mxCodecRegistry.addPackage("org.scilab.modules.metanet.edge");
    	mxCodecRegistry.addPackage("org.scilab.modules.metanet.node");
    	mxCodecRegistry.addPackage("org.scilab.modules.metanet.graph");
    	// Add some hdf5 packages to have all scilab types known
    	mxCodecRegistry.addPackage("org.scilab.modules.hdf5.scilabTypes");
    	
    	String[] refs = {"parent", "source", "target"};

    	// Types
    	ScilabObjectCodec scilabStringCodec = new ScilabStringCodec(new ScilabString(), null, null, null);
    	mxCodecRegistry.register(scilabStringCodec);
    	ScilabObjectCodec scilabBooleanCodec = new ScilabBooleanCodec(new ScilabBoolean(), null, null, null);
    	mxCodecRegistry.register(scilabBooleanCodec);
    	ScilabObjectCodec scilabDoubleCodec = new ScilabDoubleCodec(new ScilabDouble(), null, null, null);
    	mxCodecRegistry.register(scilabDoubleCodec);
    	ScilabObjectCodec scilabIntegerCodec = new ScilabIntegerCodec(new ScilabInteger(), null, null, null);
    	mxCodecRegistry.register(scilabIntegerCodec);



        //
    	ScilabObjectCodec scilabListCodec = new ScilabListCodec(new ScilabList(), new String[]{"scilabClass"}, null, null);
    	mxCodecRegistry.register(scilabListCodec);
    	
    	//
    	String[] diagramIgnore = {"stylesheet", "parentTab", "viewPort", "viewPortMenu", "view", "selectionModel", "savedFile", "multiplicities"};
    	mxObjectCodec diagramCodec = new mxObjectCodec(new MetanetDiagram(), diagramIgnore, refs, null);
    	mxCodecRegistry.register(diagramCodec);
    	
    	//
    	mxObjectCodec basicNodeCodec = new mxObjectCodec(new BasicNode(), null, refs, null);
    	mxCodecRegistry.register(basicNodeCodec);
    	
    	mxObjectCodec defaultNodeCodec = new mxObjectCodec(new DefaultNode(),  new String[]{"geometry"}, refs, null);
    	mxCodecRegistry.register( defaultNodeCodec);
    	
    	//
    	//
    	mxObjectCodec basicEdgeCodec = new mxObjectCodec(new BasicEdge(), null, refs, null);
    	mxCodecRegistry.register(basicEdgeCodec);
    }

	public MetanetCodec() {
		super();
	}

	public MetanetCodec(Document document) {
		super(document);
	}

}
